/*
  USB Desktop Bell
  2019-05-29
  T. Nakagawa

  PWM: PD5 (OC0B)
  FET: PD4
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdint.h>
#include <util/delay.h>
#include "usbdrv.h"

PROGMEM const char usbHidReportDescriptor[22] = {
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};

static struct {
  uint8_t angle1;
  uint8_t angle2;
  uint8_t angle3;
  uint8_t time3;
  uint16_t time1;
  uint16_t time2;
} request;

volatile static int8_t state;

uint8_t usbFunctionWrite(uint8_t *data, uint8_t len) {
  if (len == 8) {
    memcpy(&request, data, len);
    state = 1;
    return 1;
  } else {
    return -1;
  }
}

usbMsgLen_t usbFunctionSetup(uint8_t data[8]) {
  usbRequest_t *rq = (usbRequest_t *)data;

  if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) {
    if (rq->bRequest == USBRQ_HID_GET_REPORT) {
      usbMsgPtr = (usbMsgPtr_t)&request;
      return 8;
    } else if (rq->bRequest == USBRQ_HID_SET_REPORT) {
      return USB_NO_MSG;
    }
  }
  return 0;
}

void enable_servo() {
  PORTD |= _BV(4);	// FET on.
  TCCR0A = (_BV(COM0B1) | _BV(WGM01) | _BV(WGM00));	// Timer/Counter0, high speed 8 bit PWM, non-inverted output.
  TCCR0B = (_BV(CS02) | _BV(CS00));	// Timer/Counter0, 1/1024 prescaler.
  DDRD |= _BV(5);
}

void disable_servo() {
  TCCR0A = 0x00;
  TCCR0B = 0x00;
  DDRD &= ~_BV(5);
  PORTD &= ~_BV(4);
}

void set_servo(uint8_t val) {
  OCR0B = val;
}

int main(void) {
  uint8_t i;

  DDRD |= _BV(4);	// FET.
  PORTD &= ~_BV(4);	// FET off.
  TCCR1B = (_BV(CS12) | _BV(CS10));	// Timer/Counter1, 1/1024 prescaler.
  state = 0;

  wdt_enable(WDTO_1S);
  usbInit();
  usbDeviceDisconnect();
  for (i = 0; i < 250; i++) {  // Wait for 250ms.
    wdt_reset();
    _delay_ms(1);
  }
  usbDeviceConnect();
  sei();
  state = 0;
  for(; ; ){
    wdt_reset();
    usbPoll();
    switch (state) {
    case 1:
      set_servo(request.angle1);
      enable_servo();
      TCNT1 = 0;
      state = 2;
      break;
    case 2:
      if (TCNT1 >= request.time1) {
        state = 3;
      }
      break;
    case 3:
      set_servo(request.angle2);
      TCNT1 = 0;
      state = 4;
      break;
    case 4:
      if (TCNT1 >= request.time2) {
        state = 5;
      }
      break;
    case 5:
      set_servo(request.angle3);
      TCNT1 = 0;
      state = 6;
      break;
    case 6:
      if (TCNT1 >= (((uint16_t)request.time3) << 8)) {
        state = 7;
      }
      break;
    case 7:
      disable_servo();
      state = 0;
      break;
    }
  }
  return 0;
}
